@@ -85,7 +85,6 @@ InitThreadData(JSThreadData *data)
 #ifdef JS_TRACER
     js_InitJIT(&data->traceMonitor);
 #endif
-    js_InitRandom(data);
 }
 
 static void
@@ -487,6 +486,8 @@ js_NewContext(JSRuntime *rt, size_t stackChunkSize)
     JS_APPEND_LINK(&cx->link, &rt->contextList);
     JS_UNLOCK_GC(rt);
 
+    js_InitRandom(cx);
+
     /*
      * If cx is the first context on this runtime, initialize well-known atoms,
      * keywords, numbers, and strings.  If one of these steps should fail, the
