@@ -284,9 +284,6 @@ struct JSThreadData {
     /* Property cache for faster call/get/set invocation. */
     JSPropertyCache     propertyCache;
 
-    /* Random number generator state, used by jsmath.cpp. */
-    int64               rngSeed;
-
 #ifdef JS_TRACER
     /* Trace-tree JIT recorder/interpreter state. */
     JSTraceMonitor      traceMonitor;
@@ -956,6 +953,8 @@ typedef struct JSLocalRootStack {
 
 #define JSRESOLVE_INFER         0xffff  /* infer bits from current bytecode */
 
+extern const JSDebugHooks js_NullDebugHooks;  /* defined in jsdbgapi.cpp */
+
 struct JSContext {
     /*
      * If this flag is set, we were asked to call back the operation callback
@@ -1124,6 +1123,9 @@ struct JSContext {
     /* Stored here to avoid passing it around as a parameter. */
     uintN               resolveFlags;
 
+    /* Random number generator state, used by jsmath.cpp. */
+    int64               rngSeed;
+
 #ifdef JS_TRACER
     /*
      * State for the current tree execution.  bailExit is valid if the tree has
@@ -1152,8 +1154,9 @@ struct JSContext {
     void updateJITEnabled() {
 #ifdef JS_TRACER
         jitEnabled = ((options & JSOPTION_JIT) &&
-                      !runtime->debuggerInhibitsJIT() &&
-                      debugHooks == &runtime->globalDebugHooks);
+                      (debugHooks == &js_NullDebugHooks ||
+                       (debugHooks == &runtime->globalDebugHooks &&
+                        !runtime->debuggerInhibitsJIT())));
 #endif
     }
 
@@ -1341,6 +1344,11 @@ class JSAutoTempValueRooter
     jsval value() { return mTvr.u.value; }
     jsval *addr() { return &mTvr.u.value; }
 
+    void set(jsval v) {
+        JS_ASSERT(mTvr.count == JSTVU_SINGLE);
+        mTvr.u.value = v;
+    }
+
   protected:
     JSContext *mContext;
 
