@@ -19,7 +19,8 @@
  * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Igor Bukanov <igor@mir2.org>
+ *                 Bob Clary <bob@bclary.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,34 +36,24 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = 'regress-313952-01.js';
+gTestfile = 'regress-280844-1.js';
 
-var summary = "13.3.5.2 - root QName.uri";
-var BUGNUMBER = 313952;
-var actual = '';
-var expect = '';
+var summary = 'Uncontrolled recursion in js_MarkXML during GC';
+var BUGNUMBER = 280844;
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 START(summary);
-printStatus("This test requires TOO_MUCH_GC");
 
-var str = " foo:bar".substring(1);
-expect = new QName("  foo:bar".substring(2), "a").uri;
-
-var likeString = {
-        toString: function() {
-                var tmp = str;
-                str = null;
-                return tmp;
-        }
-};
-
-actual = new QName(likeString, "a").uri;
-
-printStatus(actual.length);
-
-printStatus(expect === actual);
+var N = 5 * 1000;
+var x = <x/>;
+for (var i = 1; i <= N; ++i) {
+    x.appendChild(<x/>);
+    x = x.x[0];
+}
+printStatus(x.toXMLString());
+gc();
 
 TEST(1, expect, actual);
-
 END();
