@@ -1,4 +1,5 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: java; tab-width:8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -15,11 +16,11 @@
  * The Original Code is JavaScript Engine testing utilities.
  *
  * The Initial Developer of the Original Code is
- * Jeff Walden <jwalden+code@mit.edu>.
+ * Mozilla Foundation.
  * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s):
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,37 +36,24 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = 'regress-350206-1.js';
+gTestfile = 'regress-354145-07.js';
 
-//-----------------------------------------------------------------------------
-var BUGNUMBER     = "350206";
-var summary = "Order of destructuring, destructuring in the presence of " +
-              "exceptions";
-var actual, expect;
+var BUGNUMBER = 354145;
+var summary = 'Immutable XML';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-/**************
- * BEGIN TEST *
- **************/
 
-var failed = "No crash";
+var a = <a><b/></a>
+try {
+  a.insertChildAfter(a.b[0], {toString: function() { throw 1; }});
+} catch (e) { }
 
-try
-{
-  var t1 = <ns2:t1 xmlns:ns2="http://bar/ns2"/>;
-  var n2 = new Namespace("http://ns2");
-  t1.@n2::a1 = "a1 from ns2";
-
-  t1.toXMLString();
-}
-catch (ex)
-{
-  failed = ex;
-}
-
-expect = "No crash";
-actual = failed;
+for each (var i in a) { }
 
 TEST(1, expect, actual);
+
+END();
