@@ -16,7 +16,7 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s): Jesse Ruderman
@@ -35,30 +35,51 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-452884-01.js';
+var gTestfile = 'regress-347306-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452884;
-var summary = 'Do not crash in switch';
-var actual = 'No Crash';
-var expect = 'No Crash';
+var BUGNUMBER = 347306;
+var summary = 'decompilation should not be O(N**2)';
+var actual = '';
+var expect = '';
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
-function test()
+var data = {X:[], Y:[]};
+for (var size = 1000; size <= 10000; size += 1000)
+{ 
+  data.X.push(size);
+  data.Y.push(testSource(size));
+  gc();
+}
+
+var order = BigO(data);
+
+var msg = '';
+for (var p = 0; p < data.X.length; p++)
+{
+  msg += '(' + data.X[p] + ', ' + data.Y[p] + '); ';
+}
+printStatus(msg);
+printStatus('Order: ' + order);
+reportCompare(true, order < 2, 'BigO ' + order + ' < 2');
+
+function testSource(n)
 {
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
+  var funtext = "";
+ 
+  for (var i=0; i<n; ++i)
+    funtext += "alert(" + i + "); ";
 
-  jit(true);
+  var fun = new Function(funtext);
 
-  for (var j=0;j<5;++j) { switch(1.1) { case NaN: case 2: } }
+  var start = new Date();
+ 
+  var s = fun + "";
  
-  jit(false);
+  var end = new Date();
 
-  reportCompare(expect, actual, summary);
+  print("Size: " + n + ", Time: " + (end - start) + " ms");
 
-  exitFunc ('test');
+  return end - start;
 }
