@@ -16,7 +16,7 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2007
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s): Jesse Ruderman
@@ -35,10 +35,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-373827-01.js';
+var gTestfile = 'regress-452498-076.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 373827;
-var summary = 'Do not assert: OBJ_GET_CLASS(cx, obj)->flags & JSCLASS_HAS_PRIVATE';
+var BUGNUMBER = 452498;
+var summary = 'TM: upvar2 regression tests';
 var actual = '';
 var expect = '';
 
@@ -52,8 +52,11 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  let ([] = [{x: function(){}}]) { };
+
+// ------- Comment #76 From Jesse Ruderman
+
+  for (let d = 0; d < 4; ++d) { d; }
+// 1: ReferenceError: d is not defined
 
   reportCompare(expect, actual, summary);
 
