@@ -53,11 +53,12 @@
 #include "nsThreadUtilsInternal.h"
 #include "dom_quickstubs.h"
 
-NS_IMPL_THREADSAFE_ISUPPORTS6(nsXPConnect,
+NS_IMPL_THREADSAFE_ISUPPORTS7(nsXPConnect,
                               nsIXPConnect,
                               nsISupportsWeakReference,
                               nsIThreadObserver,
                               nsIJSRuntimeService,
+                              nsIJSRuntimeService_MOZILLA_1_9_2,
                               nsIJSContextStack,
                               nsIThreadJSContextStack)
 
@@ -67,7 +68,7 @@ PRUint32     nsXPConnect::gReportAllJSExceptions = 0;
 
 // Global cache of the default script security manager (QI'd to
 // nsIScriptSecurityManager)
-nsIScriptSecurityManager *gScriptSecurityManager = nsnull;
+nsIScriptSecurityManager_1_9_2 *gScriptSecurityManager = nsnull;
 
 const char XPC_CONTEXT_STACK_CONTRACTID[] = "@mozilla.org/js/xpc/ContextStack;1";
 const char XPC_RUNTIME_CONTRACTID[]       = "@mozilla.org/js/xpc/RuntimeService;1";
@@ -571,26 +572,9 @@ nsXPConnect::BeginCycleCollection(nsCycleCollectionTraversalCallback &cb)
     GetRuntime()->AddXPConnectRoots(mCycleCollectionContext->GetJSContext(),
                                     cb);
 
-#ifndef XPCONNECT_STANDALONE
-    if(!mScopes.IsInitialized())
-    {
-        mScopes.Init();
-    }
-    NS_ASSERTION(mScopes.Count() == 0, "Didn't clear mScopes?");
-    XPCWrappedNativeScope::TraverseScopes(*mCycleCollectionContext);
-#endif
-
     return NS_OK;
 }
 
-#ifndef XPCONNECT_STANDALONE
-void
-nsXPConnect::RecordTraversal(void *p, nsISupports *s)
-{
-    mScopes.Put(p, s);
-}
-#endif
-
 nsresult 
 nsXPConnect::FinishCycleCollection()
 {
@@ -604,10 +588,6 @@ nsXPConnect::FinishCycleCollection()
     }
 #endif
 
-#ifndef XPCONNECT_STANDALONE
-    mScopes.Clear();
-#endif
-
 #ifdef DEBUG_CC
     if(mJSRoots.ops)
     {
@@ -625,6 +605,16 @@ nsXPConnect::ToParticipant(void *p)
     return this;
 }
 
+void
+nsXPConnect::CommenceShutdown()
+{
+#ifdef DEBUG
+    fprintf(stderr, "nsXPConnect::CommenceShutdown()\n");
+#endif
+    // Tell the JS engine that we are about to destroy the runtime.
+    JS_CommenceRuntimeShutDown(mRuntime->GetJSRuntime());
+}
+
 NS_IMETHODIMP
 nsXPConnect::RootAndUnlinkJSObjects(void *p)
 {
@@ -672,21 +662,23 @@ NoteJSChild(JSTracer *trc, void *thing, uint32 kind)
     if(ADD_TO_CC(kind))
     {
         TraversalTracer *tracer = static_cast<TraversalTracer*>(trc);
-#if defined(DEBUG) && defined(DEBUG_CC)
-        // based on DumpNotify in jsapi.c
-        if (tracer->debugPrinter) {
-            char buffer[200];
-            tracer->debugPrinter(trc, buffer, sizeof(buffer));
-            tracer->cb.NoteNextEdgeName(buffer);
-        } else if (tracer->debugPrintIndex != (size_t)-1) {
-            char buffer[200];
-            JS_snprintf(buffer, sizeof(buffer), "%s[%lu]",
-                        static_cast<const char *>(tracer->debugPrintArg),
-                        tracer->debugPrintIndex);
-            tracer->cb.NoteNextEdgeName(buffer);
-        } else {
-            tracer->cb.NoteNextEdgeName(
-              static_cast<const char*>(tracer->debugPrintArg));
+#if defined(DEBUG)
+        if (NS_UNLIKELY(tracer->cb.WantDebugInfo())) {
+            // based on DumpNotify in jsapi.c
+            if (tracer->debugPrinter) {
+                char buffer[200];
+                tracer->debugPrinter(trc, buffer, sizeof(buffer));
+                tracer->cb.NoteNextEdgeName(buffer);
+            } else if (tracer->debugPrintIndex != (size_t)-1) {
+                char buffer[200];
+                JS_snprintf(buffer, sizeof(buffer), "%s[%lu]",
+                            static_cast<const char *>(tracer->debugPrintArg),
+                            tracer->debugPrintIndex);
+                tracer->cb.NoteNextEdgeName(buffer);
+            } else {
+                tracer->cb.NoteNextEdgeName(
+                  static_cast<const char*>(tracer->debugPrintArg));
+            }
         }
 #endif
         tracer->cb.NoteScriptChild(nsIProgrammingLanguage::JAVASCRIPT, thing);
@@ -753,7 +745,6 @@ nsXPConnect::Traverse(void *p, nsCycleCollectionTraversalCallback &cb)
     CCNodeType type;
 
 #ifdef DEBUG_CC
-    {
     // Note that the conditions under which we specify GCMarked vs.
     // GCUnmarked are different between ExplainLiveExpectedGarbage and
     // the normal case.  In the normal case, we're saying that anything
@@ -773,149 +764,150 @@ nsXPConnect::Traverse(void *p, nsCycleCollectionTraversalCallback &cb)
                                                                GCUnmarked;
     }
     else
+#endif
     {
         // Normal codepath (matches non-DEBUG_CC codepath).
         type = !markJSObject && JS_IsAboutToBeFinalized(cx, p) ? GCUnmarked :
                                                                  GCMarked;
     }
 
-    char name[72];
-    if(traceKind == JSTRACE_OBJECT)
-    {
-        JSObject *obj = static_cast<JSObject*>(p);
-        JSClass *clazz = OBJ_GET_CLASS(cx, obj);
-        if(XPCNativeWrapper::IsNativeWrapperClass(clazz))
+    if (cb.WantDebugInfo()) {
+        char name[72];
+        if(traceKind == JSTRACE_OBJECT)
         {
-            XPCWrappedNative* wn = XPCNativeWrapper::GetWrappedNative(obj);
-            if(wn)
+            JSObject *obj = static_cast<JSObject*>(p);
+            JSClass *clazz = OBJ_GET_CLASS(cx, obj);
+            if(XPCNativeWrapper::IsNativeWrapperClass(clazz))
             {
-                XPCNativeScriptableInfo* si = wn->GetScriptableInfo();
-                if(si)
+                XPCWrappedNative* wn;
+                if(XPCNativeWrapper::GetWrappedNative(cx, obj, &wn) && wn)
                 {
-                    JS_snprintf(name, sizeof(name), "XPCNativeWrapper (%s)",
-                                si->GetJSClass()->name);
-                }
-                else
-                {
-                    nsIClassInfo* ci = wn->GetClassInfo();
-                    char* className = nsnull;
-                    if(ci)
-                        ci->GetClassDescription(&className);
-                    if(className)
+                    XPCNativeScriptableInfo* si = wn->GetScriptableInfo();
+                    if(si)
                     {
                         JS_snprintf(name, sizeof(name), "XPCNativeWrapper (%s)",
-                                    className);
-                        PR_Free(className);
+                                    si->GetJSClass()->name);
                     }
                     else
                     {
-                        XPCNativeSet* set = wn->GetSet();
-                        XPCNativeInterface** array = set->GetInterfaceArray();
-                        PRUint16 count = set->GetInterfaceCount();
-
-                        if(count > 0)
+                        nsIClassInfo* ci = wn->GetClassInfo();
+                        char* className = nsnull;
+                        if(ci)
+                            ci->GetClassDescription(&className);
+                        if(className)
+                        {
                             JS_snprintf(name, sizeof(name),
-                                        "XPCNativeWrapper (%s)",
-                                        array[0]->GetNameString());
+                                        "XPCNativeWrapper (%s)", className);
+                            PR_Free(className);
+                        }
                         else
-                            JS_snprintf(name, sizeof(name), "XPCNativeWrapper");
+                        {
+                            XPCNativeSet* set = wn->GetSet();
+                            XPCNativeInterface** array =
+                                set->GetInterfaceArray();
+                            PRUint16 count = set->GetInterfaceCount();
+
+                            if(count > 0)
+                                JS_snprintf(name, sizeof(name),
+                                            "XPCNativeWrapper (%s)",
+                                            array[0]->GetNameString());
+                            else
+                                JS_snprintf(name, sizeof(name),
+                                            "XPCNativeWrapper");
+                        }
                     }
                 }
+                else
+                {
+                    JS_snprintf(name, sizeof(name), "XPCNativeWrapper");
+                }
             }
             else
             {
-                JS_snprintf(name, sizeof(name), "XPCNativeWrapper");
+                XPCNativeScriptableInfo* si = nsnull;
+                if(IS_PROTO_CLASS(clazz))
+                {
+                    XPCWrappedNativeProto* p =
+                        (XPCWrappedNativeProto*) xpc_GetJSPrivate(obj);
+                    si = p->GetScriptableInfo();
+                }
+                if(si)
+                {
+                    JS_snprintf(name, sizeof(name), "JS Object (%s - %s)",
+                                clazz->name, si->GetJSClass()->name);
+                }
+                else if(clazz == &js_ScriptClass)
+                {
+                    JSScript* script = (JSScript*) xpc_GetJSPrivate(obj);
+                    if(script->filename)
+                    {
+                        JS_snprintf(name, sizeof(name),
+                                    "JS Object (Script - %s)",
+                                    script->filename);
+                    }
+                    else
+                    {
+                        JS_snprintf(name, sizeof(name), "JS Object (Script)");
+                    }
+                }
+                else if(clazz == &js_FunctionClass)
+                {
+                    JSFunction* fun = (JSFunction*) xpc_GetJSPrivate(obj);
+                    JSString* str = JS_GetFunctionId(fun);
+                    if(str)
+                    {
+                        NS_ConvertUTF16toUTF8
+                            fname(reinterpret_cast<const PRUnichar*>(JS_GetStringChars(str)));
+                        JS_snprintf(name, sizeof(name),
+                                    "JS Object (Function - %s)", fname.get());
+                    }
+                    else
+                    {
+                        JS_snprintf(name, sizeof(name), "JS Object (Function)");
+                    }
+                }
+                else
+                {
+                    JS_snprintf(name, sizeof(name), "JS Object (%s)",
+                                clazz->name);
+                }
             }
         }
         else
         {
-            XPCNativeScriptableInfo* si = nsnull;
-            if(IS_PROTO_CLASS(clazz))
-            {
-                XPCWrappedNativeProto* p =
-                    (XPCWrappedNativeProto*) xpc_GetJSPrivate(obj);
-                si = p->GetScriptableInfo();
-            }
-            if(si)
-            {
-                JS_snprintf(name, sizeof(name), "JS Object (%s - %s)",
-                            clazz->name, si->GetJSClass()->name);
-            }
-            else if(clazz == &js_ScriptClass)
-            {
-                JSScript* script = (JSScript*) xpc_GetJSPrivate(obj);
-                if(script->filename)
-                {
-                    JS_snprintf(name, sizeof(name), "JS Object (Script - %s)",
-                                script->filename);
-                }
-                else
-                {
-                    JS_snprintf(name, sizeof(name), "JS Object (Script)");
-                }
-            }
-            else if(clazz == &js_FunctionClass)
-            {
-                JSFunction* fun = (JSFunction*) xpc_GetJSPrivate(obj);
-                JSString* str = JS_GetFunctionId(fun);
-                if(str)
-                {
-                    NS_ConvertUTF16toUTF8
-                        fname(JS_GetStringChars(str));
-                    JS_snprintf(name, sizeof(name), "JS Object (Function - %s)",
-                                fname.get());
-                }
-                else
-                {
-                    JS_snprintf(name, sizeof(name), "JS Object (Function)");
-                }
-            }
-            else
-            {
-                JS_snprintf(name, sizeof(name), "JS Object (%s)", clazz->name);
-            }
+            static const char trace_types[JSTRACE_LIMIT][7] = {
+                "Object",
+                "Double",
+                "String",
+                "Xml"
+            };
+            JS_snprintf(name, sizeof(name), "JS %s", trace_types[traceKind]);
         }
-    }
-    else
-    {
-        static const char trace_types[JSTRACE_LIMIT][7] = {
-            "Object",
-            "Double",
-            "String",
-            "Xml"
-        };
-        JS_snprintf(name, sizeof(name), "JS %s", trace_types[traceKind]);
-    }
 
-    if(traceKind == JSTRACE_OBJECT) {
-        JSObject *global = static_cast<JSObject*>(p), *parent;
-        while((parent = JS_GetParent(cx, global)))
-            global = parent;
-        char fullname[100];
-        JS_snprintf(fullname, sizeof(fullname), "%s (global=%p)", name, global);
-        cb.DescribeNode(type, 0, sizeof(JSObject), fullname);
+        if(traceKind == JSTRACE_OBJECT) {
+            JSObject *global = static_cast<JSObject*>(p), *parent;
+            while((parent = JS_GetParent(cx, global)))
+                global = parent;
+            char fullname[100];
+            JS_snprintf(fullname, sizeof(fullname),
+                        "%s (global=%p)", name, global);
+            cb.DescribeNode(type, 0, sizeof(JSObject), fullname);
+        } else {
+            cb.DescribeNode(type, 0, sizeof(JSObject), name);
+        }
     } else {
-        cb.DescribeNode(type, 0, sizeof(JSObject), name);
+        cb.DescribeNode(type, 0, sizeof(JSObject), "JS Object");
     }
 
-    }
-#else
-    type = !markJSObject && JS_IsAboutToBeFinalized(cx, p) ? GCUnmarked :
-                                                             GCMarked;
-    cb.DescribeNode(type, 0);
-#endif
-
     if(!ADD_TO_CC(traceKind))
         return NS_OK;
 
-#ifndef DEBUG_CC
     // There's no need to trace objects that have already been marked by the JS
     // GC. Any JS objects hanging from them will already be marked. Only do this
     // if DEBUG_CC is not defined, else we do want to know about all JS objects
     // to get better graphs and explanations.
-    if(type == GCMarked)
+    if(!cb.WantAllTraces() && type == GCMarked)
         return NS_OK;
-#endif
 
     TraversalTracer trc(cb);
 
@@ -947,16 +939,6 @@ nsXPConnect::Traverse(void *p, nsCycleCollectionTraversalCallback &cb)
         cb.NoteXPCOMChild(static_cast<nsISupports*>(xpc_GetJSPrivate(obj)));
     }
 
-#ifndef XPCONNECT_STANDALONE
-    if(clazz->flags & JSCLASS_IS_GLOBAL)
-    {
-        nsISupports *principal = nsnull;
-        mScopes.Get(obj, &principal);
-        NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "scope principal");
-        cb.NoteXPCOMChild(principal);
-    }
-#endif
-
     return NS_OK;
 }
 
@@ -999,13 +981,9 @@ public:
         // collected.
         PRInt32 refCount = nsXPConnect::GetXPConnect()->GetRequestDepth(cx) + 1;
 
-#ifdef DEBUG_CC
         cb.DescribeNode(RefCounted, refCount, sizeof(JSContext),
                         "JSContext");
-        cb.NoteNextEdgeName("[global object]");
-#else
-        cb.DescribeNode(RefCounted, refCount);
-#endif
+        NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "[global object]");
         cb.NoteScriptChild(nsIProgrammingLanguage::JAVASCRIPT,
                            cx->globalObject);
 
@@ -1086,16 +1064,14 @@ nsXPConnect::InitClasses(JSContext * aJSContext, JSObject * aGlobalJSObj)
     if(!nsXPCComponents::AttachNewComponentsObject(ccx, scope, aGlobalJSObj))
         return UnexpectedFailure(NS_ERROR_FAILURE);
 
-#ifdef XPC_IDISPATCH_SUPPORT
-    // Initialize any properties IDispatch needs on the global object
-    XPCIDispatchExtension::Initialize(ccx, aGlobalJSObj);
-#endif
+    if(XPCPerThreadData::IsMainThread(ccx))
+    {
+        if (!XPCNativeWrapper::AttachNewConstructorObject(ccx, aGlobalJSObj))
+            return UnexpectedFailure(NS_ERROR_FAILURE);
 
-    if (!XPCNativeWrapper::AttachNewConstructorObject(ccx, aGlobalJSObj))
-        return UnexpectedFailure(NS_ERROR_FAILURE);
-
-    if (!XPC_SJOW_AttachNewConstructorObject(ccx, aGlobalJSObj))
-        return UnexpectedFailure(NS_ERROR_FAILURE);
+        if (!XPC_SJOW_AttachNewConstructorObject(ccx, aGlobalJSObj))
+            return UnexpectedFailure(NS_ERROR_FAILURE);
+    }
 
     return NS_OK;
 }
@@ -1110,7 +1086,7 @@ TempGlobalResolve(JSContext *aJSContext, JSObject *obj, jsval id)
 static JSClass xpcTempGlobalClass = {
     "xpcTempGlobalClass", JSCLASS_GLOBAL_FLAGS,
     JS_PropertyStub,  JS_PropertyStub,  JS_PropertyStub,  JS_PropertyStub,
-    JS_EnumerateStub, TempGlobalResolve, JS_ConvertStub,   JS_FinalizeStub,
+    JS_EnumerateStub, TempGlobalResolve, JS_ConvertStub,   nsnull,
     JSCLASS_NO_OPTIONAL_MEMBERS
 };
 
@@ -1208,11 +1184,14 @@ nsXPConnect::InitClassesWithNewWrappedGlobal(JSContext * aJSContext,
         if(!nsXPCComponents::AttachNewComponentsObject(ccx, scope, globalJSObj))
             return UnexpectedFailure(NS_ERROR_FAILURE);
 
-        if(!XPCNativeWrapper::AttachNewConstructorObject(ccx, globalJSObj))
-            return UnexpectedFailure(NS_ERROR_FAILURE);
+        if(XPCPerThreadData::IsMainThread(ccx))
+        {
+            if(!XPCNativeWrapper::AttachNewConstructorObject(ccx, globalJSObj))
+                return UnexpectedFailure(NS_ERROR_FAILURE);
 
-        if(!XPC_SJOW_AttachNewConstructorObject(ccx, globalJSObj))
-            return UnexpectedFailure(NS_ERROR_FAILURE);
+            if(!XPC_SJOW_AttachNewConstructorObject(ccx, globalJSObj))
+                return UnexpectedFailure(NS_ERROR_FAILURE);
+        }
     }
 
     NS_ADDREF(*_retval = holder);
@@ -1231,7 +1210,8 @@ nsXPConnect::WrapNative(JSContext * aJSContext,
     NS_ASSERTION(aHolder, "bad param");
 
     jsval v;
-    return WrapNativeToJSVal(aJSContext, aScope, aCOMObj, &aIID, &v, aHolder);
+    return WrapNativeToJSVal(aJSContext, aScope, aCOMObj, &aIID, PR_FALSE,
+                             &v, aHolder);
 }
 
 /* void wrapNativeToJSVal (in JSContextPtr aJSContext, in JSObjectPtr aScope, in nsISupports aCOMObj, in nsIIDPtr aIID, out JSVal aVal, out nsIXPConnectJSObjectHolder aHolder); */
@@ -1240,6 +1220,7 @@ nsXPConnect::WrapNativeToJSVal(JSContext * aJSContext,
                                JSObject * aScope,
                                nsISupports *aCOMObj,
                                const nsIID * aIID,
+                               PRBool aAllowWrapping,
                                jsval *aVal,
                                nsIXPConnectJSObjectHolder **aHolder)
 {
@@ -1253,10 +1234,11 @@ nsXPConnect::WrapNativeToJSVal(JSContext * aJSContext,
     XPCCallContext ccx(NATIVE_CALLER, aJSContext);
     if(!ccx.IsValid())
         return UnexpectedFailure(NS_ERROR_FAILURE);
+    XPCLazyCallContext lccx(ccx);
 
     nsresult rv;
     if(!XPCConvert::NativeInterface2JSObject(ccx, aVal, aHolder, aCOMObj, aIID,
-                                             nsnull, nsnull, aScope, PR_FALSE,
+                                             nsnull, nsnull, aScope, aAllowWrapping,
                                              OBJ_IS_NOT_GLOBAL, &rv))
         return rv;
 
@@ -1332,12 +1314,74 @@ nsXPConnect::GetWrappedNativeOfJSObject(JSContext * aJSContext,
     if(!ccx.IsValid())
         return UnexpectedFailure(NS_ERROR_FAILURE);
 
+    SLIM_LOG_WILL_MORPH(aJSContext, aJSObj);
+    if(!IS_SLIM_WRAPPER(aJSObj) || MorphSlimWrapper(aJSContext, aJSObj))
+    {
+        nsIXPConnectWrappedNative* wrapper =
+            XPCWrappedNative::GetWrappedNativeOfJSObject(aJSContext, aJSObj);
+        if(wrapper)
+        {
+            NS_ADDREF(wrapper);
+            *_retval = wrapper;
+            return NS_OK;
+        }
+    }
+
+    // else...
+    *_retval = nsnull;
+    return NS_ERROR_FAILURE;
+}
+
+/* nsISupports getNativeOfWrapper(in JSContextPtr aJSContext, in JSObjectPtr  aJSObj); */
+NS_IMETHODIMP_(nsISupports*)
+nsXPConnect::GetNativeOfWrapper(JSContext * aJSContext,
+                                JSObject * aJSObj)
+{
+    NS_ASSERTION(aJSContext, "bad param");
+    NS_ASSERTION(aJSObj, "bad param");
+
+    XPCCallContext ccx(NATIVE_CALLER, aJSContext);
+    if(!ccx.IsValid())
+    {
+        UnexpectedFailure(NS_ERROR_FAILURE);
+        return nsnull;
+    }
+
+    JSObject* obj2 = nsnull;
     nsIXPConnectWrappedNative* wrapper =
-        XPCWrappedNative::GetWrappedNativeOfJSObject(aJSContext, aJSObj);
+        XPCWrappedNative::GetWrappedNativeOfJSObject(aJSContext, aJSObj, nsnull,
+                                                     &obj2);
+
+    return wrapper ? wrapper->Native() :
+                     (obj2 ? (nsISupports*)xpc_GetJSPrivate(obj2) : nsnull);
+}
+
+/* JSObjectPtr getJSObjectOfWrapper (in JSContextPtr aJSContext, in JSObjectPtr aJSObj); */
+NS_IMETHODIMP
+nsXPConnect::GetJSObjectOfWrapper(JSContext * aJSContext,
+                                  JSObject * aJSObj,
+                                  JSObject **_retval)
+{
+    NS_ASSERTION(aJSContext, "bad param");
+    NS_ASSERTION(aJSObj, "bad param");
+    NS_ASSERTION(_retval, "bad param");
+
+    XPCCallContext ccx(NATIVE_CALLER, aJSContext);
+    if(!ccx.IsValid())
+        return UnexpectedFailure(NS_ERROR_FAILURE);
+
+    JSObject* obj2 = nsnull;
+    nsIXPConnectWrappedNative* wrapper =
+        XPCWrappedNative::GetWrappedNativeOfJSObject(aJSContext, aJSObj, nsnull,
+                                                     &obj2);
     if(wrapper)
     {
-        NS_ADDREF(wrapper);
-        *_retval = wrapper;
+        wrapper->GetJSObject(_retval);
+        return NS_OK;
+    }
+    if(obj2)
+    {
+        *_retval = obj2;
         return NS_OK;
     }
     // else...
@@ -1416,16 +1460,17 @@ MoveableWrapperFinder(JSDHashTable *table, JSDHashEntryHdr *hdr,
                       uint32 number, void *arg)
 {
     // Every element counts.
-    nsVoidArray *va = static_cast<nsVoidArray *>(arg);
-    va->AppendElement(((Native2WrappedNativeMap::Entry*)hdr)->value);
+    nsTArray<nsRefPtr<XPCWrappedNative> > *array =
+        static_cast<nsTArray<nsRefPtr<XPCWrappedNative> > *>(arg);
+    array->AppendElement(((Native2WrappedNativeMap::Entry*)hdr)->value);
     return JS_DHASH_NEXT;
 }
 
-/* void reparentScopeAwareWrappers(in JSContextPtr aJSContext, in JSObjectPtr  aOldScope, in JSObjectPtr  aNewScope); */
+/* void moveWrappers(in JSContextPtr aJSContext, in JSObjectPtr  aOldScope, in JSObjectPtr  aNewScope); */
 NS_IMETHODIMP
-nsXPConnect::ReparentScopeAwareWrappers(JSContext *aJSContext,
-                                        JSObject *aOldScope,
-                                        JSObject *aNewScope)
+nsXPConnect::MoveWrappers(JSContext *aJSContext,
+                          JSObject *aOldScope,
+                          JSObject *aNewScope)
 {
     XPCCallContext ccx(NATIVE_CALLER, aJSContext);
     if(!ccx.IsValid())
@@ -1443,23 +1488,22 @@ nsXPConnect::ReparentScopeAwareWrappers(JSContext *aJSContext,
 
     // First, look through the old scope and find all of the wrappers that
     // we're going to move.
-    nsVoidArray wrappersToMove;
+    nsTArray<nsRefPtr<XPCWrappedNative> > wrappersToMove;
 
     {   // scoped lock
         XPCAutoLock lock(GetRuntime()->GetMapLock());
         Native2WrappedNativeMap *map = oldScope->GetWrappedNativeMap();
-        wrappersToMove.SizeTo(map->Count());
+        wrappersToMove.SetCapacity(map->Count());
         map->Enumerate(MoveableWrapperFinder, &wrappersToMove);
     }
 
     // Now that we have the wrappers, reparent them to the new scope.
-    for(PRInt32 i = 0, stop = wrappersToMove.Count(); i < stop; ++i)
+    for(PRUint32 i = 0, stop = wrappersToMove.Length(); i < stop; ++i)
     {
         // First, check to see if this wrapper really needs to be
         // reparented.
 
-        XPCWrappedNative *wrapper =
-            static_cast<XPCWrappedNative *>(wrappersToMove[i]);
+        XPCWrappedNative *wrapper = wrappersToMove[i];
         nsISupports *identity = wrapper->GetIdentityObject();
         nsCOMPtr<nsIClassInfo> info(do_QueryInterface(identity));
 
@@ -1494,25 +1538,30 @@ nsXPConnect::ReparentScopeAwareWrappers(JSContext *aJSContext,
         if(NS_FAILED(rv))
             return rv;
 
-        if(newParent != aOldScope)
+        if(newParent == aOldScope)
         {
-            // The wrapper returned a new parent. If the new parent is in
-            // a different scope, then we need to reparent it, otherwise,
-            // the old scope is fine.
-
-            XPCWrappedNativeScope *betterScope =
-                XPCWrappedNativeScope::FindInJSObjectScope(ccx, newParent);
-            if(betterScope == oldScope)
-                continue;
-
-            NS_ASSERTION(betterScope == newScope, "Weird scope returned");
-        }
-        else
-        {
-            // The old scope still works for this wrapper.
+            // The old scope still works for this wrapper. We have to assume
+            // that the wrapper will continue to return the old scope from
+            // PreCreate, so don't move it.
             continue;
         }
 
+        // The wrapper returned a new parent. If the new parent is in
+        // a different scope, then we need to reparent it, otherwise,
+        // the old scope is fine.
+
+        XPCWrappedNativeScope *betterScope =
+            XPCWrappedNativeScope::FindInJSObjectScope(ccx, newParent);
+        if(betterScope == oldScope)
+        {
+            // The wrapper asked for a different object, but that object
+            // was in the same scope. We assume here that the new parent
+            // simply hasn't been reparented yet.
+            newParent = nsnull;
+        }
+        else
+            NS_ASSERTION(betterScope == newScope, "Weird scope returned");
+
         // Now, reparent the wrapper, since we know that it wants to be
         // reparented.
 
@@ -1583,7 +1632,7 @@ nsXPConnect::SetDefaultSecurityManager(nsIXPCSecurityManager *aManager,
     mDefaultSecurityManager = aManager;
     mDefaultSecurityManagerFlags = flags;
 
-    nsCOMPtr<nsIScriptSecurityManager> ssm =
+    nsCOMPtr<nsIScriptSecurityManager_1_9_2> ssm =
         do_QueryInterface(mDefaultSecurityManager);
 
     // Remember the result of the above QI for fast access to the
@@ -1816,7 +1865,8 @@ nsXPConnect::RestoreWrappedNativePrototype(JSContext * aJSContext,
     if(si && si->GetFlags().DontSharePrototype())
         return UnexpectedFailure(NS_ERROR_INVALID_ARG);
 
-    ClassInfo2WrappedNativeProtoMap* map = scope->GetWrappedNativeProtoMap();
+    ClassInfo2WrappedNativeProtoMap* map =
+        scope->GetWrappedNativeProtoMap(proto->ClassIsMainThreadOnly());
     XPCLock* lock = GetRuntime()->GetMapLock();
 
     {   // scoped lock
@@ -1915,19 +1965,16 @@ nsXPConnect::EvalInSandboxObject(const nsAString& source, JSContext *cx,
 #endif /* XPCONNECT_STANDALONE */
 }
 
-/* void GetXPCWrappedNativeJSClassInfo(out JSClassConstPtr clazz, out JSObjectOpsConstPtr ops1, out JSObjectOpsConstPtr ops2); */
+/* void GetXPCWrappedNativeJSClassInfo(out JSEqualityOp equality1, JSEqualityOp *equality2); */
 NS_IMETHODIMP
-nsXPConnect::GetXPCWrappedNativeJSClassInfo(const JSClass **clazz,
-                                            JSGetObjectOps *ops1,
-                                            JSGetObjectOps *ops2)
+nsXPConnect::GetXPCWrappedNativeJSClassInfo(JSEqualityOp *equality1,
+                                            JSEqualityOp *equality2)
 {
-    // Expose the JSClass and JSGetObjectOps pointers used by
-    // IS_WRAPPER_CLASS(). If that macro ever changes, this function
-    // needs to stay in sync.
+    // Expose the equality pointer used by IS_WRAPPER_CLASS(). If that macro
+    // ever changes, this function needs to stay in sync.
 
-    *clazz = &XPC_WN_NoHelper_JSClass.base;
-    *ops1 = XPC_WN_GetObjectOpsNoCall;
-    *ops2 = XPC_WN_GetObjectOpsWithCall;
+    *equality1 = &XPC_WN_Equality;
+    *equality2 = &XPC_SWN_Equality;
 
     return NS_OK;
 }
@@ -1980,6 +2027,17 @@ nsXPConnect::GetXOWForObject(JSContext * aJSContext,
            ? NS_OK : NS_ERROR_FAILURE;
 }
 
+NS_IMETHODIMP
+nsXPConnect::GetCOWForObject(JSContext * aJSContext,
+                             JSObject * aParent,
+                             JSObject * aWrappedObj,
+                             jsval * rval)
+{
+    *rval = OBJECT_TO_JSVAL(aWrappedObj);
+    return XPC_COW_WrapObject(aJSContext, aParent, *rval, rval)
+           ? NS_OK : NS_ERROR_FAILURE;
+}
+
 static inline PRBool
 PerformOp(JSContext *cx, PRUint32 aWay, JSObject *obj)
 {
@@ -2198,9 +2256,10 @@ nsXPConnect::VariantToJS(JSContext* ctx, JSObject* scope, nsIVariant* value, jsv
     XPCCallContext ccx(NATIVE_CALLER, ctx);
     if(!ccx.IsValid())
         return NS_ERROR_FAILURE;
+    XPCLazyCallContext lccx(ccx);
 
     nsresult rv = NS_OK;
-    if(!XPCVariant::VariantDataToJS(ccx, value, scope, &rv, _retval))
+    if(!XPCVariant::VariantDataToJS(lccx, value, scope, &rv, _retval))
     {
         if(NS_FAILED(rv)) 
             return rv;
@@ -2319,18 +2378,28 @@ nsXPConnect::GetWrapperForObject(JSContext* aJSContext,
 
     JSAutoRequest ar(aJSContext);
 
-    XPCWrappedNative *wrapper =
-        XPCWrappedNative::GetWrappedNativeOfJSObject(aJSContext, aObject);
-    if(!wrapper)
+    XPCWrappedNativeScope *objectscope;
+    XPCWrappedNative *wrapper = nsnull;
+    if(IS_SLIM_WRAPPER(aObject))
     {
-        // Couldn't get the wrapped native (maybe a prototype?) so just return
-        // the original object.
-        *_retval = OBJECT_TO_JSVAL(aObject);
-        return NS_OK;
+        objectscope = GetSlimWrapperProto(aObject)->GetScope();
+    }
+    else
+    {
+        wrapper =
+            XPCWrappedNative::GetWrappedNativeOfJSObject(aJSContext, aObject);
+        if(!wrapper)
+        {
+            // Couldn't get the wrapped native (maybe a prototype?) so just return
+            // the original object.
+            *_retval = OBJECT_TO_JSVAL(aObject);
+            return NS_OK;
+        }
+        objectscope = wrapper->GetScope();
     }
 
     XPCWrappedNativeScope *xpcscope =
-        XPCWrappedNativeScope::FindInJSObjectScope(aJSContext, aScope);
+            XPCWrappedNativeScope::FindInJSObjectScope(aJSContext, aScope);
     if(!xpcscope)
         return NS_ERROR_FAILURE;
 
@@ -2343,7 +2412,6 @@ nsXPConnect::GetWrapperForObject(JSContext* aJSContext,
     }
 #endif
 
-    XPCWrappedNativeScope *objectscope = wrapper->GetScope();
     {
         JSObject *possibleOuter = objectscope->GetGlobalJSObject();
         OBJ_TO_INNER_OBJECT(aJSContext, possibleOuter);
@@ -2372,11 +2440,20 @@ nsXPConnect::GetWrapperForObject(JSContext* aJSContext,
     //   - We're not about to force a XOW (e.g. for "window") OR
     //   - We're not actually going to create a XOW (we're wrapping for
     //     chrome).
-    if(STOBJ_IS_SYSTEM(aObject) ||
+    if(aObject->isSystem() ||
        (sameScope &&
         (!forceXOW || (aFilenameFlags & JSFILENAME_SYSTEM))))
         return NS_OK;
 
+    if(!wrapper)
+    {
+        SLIM_LOG_WILL_MORPH(aJSContext, aObject);
+        if(!MorphSlimWrapper(aJSContext, aObject))
+            return NS_ERROR_FAILURE;
+
+        wrapper = static_cast<XPCWrappedNative*>(xpc_GetJSPrivate(aObject));
+    }
+
     JSObject* wrappedObj = nsnull;
 
     if(aFilenameFlags & JSFILENAME_PROTECTED)
@@ -2448,6 +2525,20 @@ nsXPConnect::GetBackstagePass(nsIXPCScriptable **bsp)
     return NS_OK;
 }
 
+/* [noscript, notxpcom] void registerGCCallback(in JSGCCallback func); */
+NS_IMETHODIMP_(void)
+nsXPConnect::RegisterGCCallback(JSGCCallback func)
+{
+    mRuntime->AddGCCallback(func);
+}
+
+/* [noscript, notxpcom] void unregisterGCCallback(in JSGCCallback func); */
+NS_IMETHODIMP_(void)
+nsXPConnect::UnregisterGCCallback(JSGCCallback func)
+{
+    mRuntime->RemoveGCCallback(func);
+}
+
 //  nsIJSContextStack and nsIThreadJSContextStack implementations
 
 /* readonly attribute PRInt32 Count; */
@@ -2543,6 +2634,58 @@ nsXPConnect::SetSafeJSContext(JSContext * aSafeJSContext)
     return data->GetJSContextStack()->SetSafeJSContext(aSafeJSContext);
 }
 
+nsIPrincipal*
+nsXPConnect::GetPrincipal(JSObject* obj, PRBool allowShortCircuit) const
+{
+    NS_ASSERTION(IS_WRAPPER_CLASS(STOBJ_GET_CLASS(obj)) || IS_SLIM_WRAPPER(obj),
+                 "What kind of wrapper is this?");
+
+    if(IS_WRAPPER_CLASS(STOBJ_GET_CLASS(obj)))
+    {
+        XPCWrappedNative *xpcWrapper =
+            (XPCWrappedNative *)xpc_GetJSPrivate(obj);
+        if (xpcWrapper) {
+            if (allowShortCircuit) {
+                nsIPrincipal *result = xpcWrapper->GetObjectPrincipal();
+                if (result) {
+                    return result;
+                }
+            }
+
+            // If not, check if it points to an nsIScriptObjectPrincipal
+            nsCOMPtr<nsIScriptObjectPrincipal> objPrin =
+                do_QueryInterface(xpcWrapper->Native());
+            if (objPrin) {
+                nsIPrincipal *result = objPrin->GetPrincipal();
+                if (result) {
+                    return result;
+                }
+            }
+        }
+    }
+    else
+    {
+        if (allowShortCircuit) {
+            nsIPrincipal *result =
+                GetSlimWrapperProto(obj)->GetScope()->GetPrincipal();
+            if (result) {
+                return result;
+            }
+        }
+
+        nsCOMPtr<nsIScriptObjectPrincipal> objPrin =
+            do_QueryInterface((nsISupports*)xpc_GetJSPrivate(obj));
+        if (objPrin) {
+            nsIPrincipal *result = objPrin->GetPrincipal();
+            if (result) {
+                return result;
+            }
+        }
+    }
+
+    return nsnull;
+}
+
 /* These are here to be callable from a debugger */
 JS_BEGIN_EXTERN_C
 JS_EXPORT_API(void) DumpJSStack()
