@@ -1248,11 +1248,12 @@ public:
   }
 #endif
 
-  virtual nsIFrame* HitTest(nsDisplayListBuilder* aBuilder, nsPoint aPt,
-                            HitTestState* aState) {
+  virtual void HitTest(nsDisplayListBuilder* aBuilder, nsRect aRect,
+                       HitTestState* aState, nsTArray<nsIFrame*> *aOutFrames) {
+    nsPoint rectCenter(aRect.x + aRect.width / 2, aRect.y + aRect.height / 2);
     static_cast<nsBoxFrame*>(mFrame)->
-      DisplayDebugInfoFor(this, aPt - aBuilder->ToReferenceFrame(mFrame));
-    return PR_TRUE;
+      DisplayDebugInfoFor(this, rectCenter - aBuilder->ToReferenceFrame(mFrame));
+    aOutFrames->AppendElement(this);
   }
   virtual void Paint(nsDisplayListBuilder* aBuilder
                      nsIRenderingContext* aCtx);
@@ -2168,31 +2169,39 @@ public:
   nsDisplayXULEventRedirector(nsIFrame* aFrame, nsDisplayList* aList,
                               nsIFrame* aTargetFrame)
     : nsDisplayWrapList(aFrame, aList), mTargetFrame(aTargetFrame) {}
-  virtual nsIFrame* HitTest(nsDisplayListBuilder* aBuilder, nsPoint aPt,
-                            HitTestState* aState);
+  virtual void HitTest(nsDisplayListBuilder* aBuilder, const nsRect& aRect,
+                       HitTestState* aState, nsTArray<nsIFrame*> *aOutFrames);
   NS_DISPLAY_DECL_NAME("XULEventRedirector")
 private:
   nsIFrame* mTargetFrame;
 };
 
-nsIFrame* nsDisplayXULEventRedirector::HitTest(nsDisplayListBuilder* aBuilder,
-    nsPoint aPt, HitTestState* aState)
+void nsDisplayXULEventRedirector::HitTest(nsDisplayListBuilder* aBuilder,
+    const nsRect& aRect, HitTestState* aState, nsTArray<nsIFrame*> *aOutFrames)
 {
-  nsIFrame* frame = mList.HitTest(aBuilder, aPt, aState);
-  if (!frame)
-    return nsnull;
+  nsTArray<nsIFrame*> outFrames;
+  mList.HitTest(aBuilder, aRect, aState, &outFrames);
 
-  for (nsIContent* content = frame->GetContent();
-       content && content != mTargetFrame->GetContent();
-       content = content->GetParent()) {
-    if (content->AttrValueIs(kNameSpaceID_None, nsGkAtoms::allowevents,
-                             nsGkAtoms::_true, eCaseMatters)) {
-      // Events are allowed on 'frame', so let it go.
-      return frame;
+  PRInt32 originalLength = aOutFrames->Length();
+  PRInt32 localLength = outFrames.Length();
+
+  for (PRUint32 i = 0; i < localLength; i++) {
+
+    for (nsIContent* content = outFrames.ElementAt(i)->GetContent();
+         content && content != mTargetFrame->GetContent();
+         content = content->GetParent()) {
+      if (content->AttrValueIs(kNameSpaceID_None, nsGkAtoms::allowevents,
+                               nsGkAtoms::_true, eCaseMatters)) {
+        // Events are allowed on 'frame', so let it go.
+        aOutFrames->AppendElement(outFrames.ElementAt(i));
+      }
     }
+
+  }
+  // If no hits were found, treat it as a hit on the target frame itself
+  if (aOutFrames->Length() == originalLength) {
+    aOutFrames->AppendElement(mTargetFrame);
   }
-  // Treat it as a hit on the target frame itself.
-  return mTargetFrame;
 }
 
 class nsXULEventRedirectorWrapper : public nsDisplayWrapper
