@@ -36,6 +36,8 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include <dlfcn.h>
+
 #include "nsMenuX.h"
 #include "nsMenuItemX.h"
 #include "nsMenuUtilsX.h"
@@ -79,7 +81,6 @@ static PRBool gMenuMethodsSwizzled = PR_FALSE;
 
 PRInt32 nsMenuX::sIndexingMenuLevel = 0;
 
-
 nsMenuX::nsMenuX()
 : mVisibleItemsCount(0), mParent(nsnull), mMenuBar(nsnull),
   mNativeMenu(nil), mNativeMenuItem(nil), mIsEnabled(PR_TRUE),
@@ -94,6 +95,12 @@ nsMenuX::nsMenuX()
                                 @selector(nsMenuX_NSMenu_addItem:toTable:), PR_TRUE);
       nsToolkit::SwizzleMethods([NSMenu class], @selector(_removeItem:fromTable:),
                                 @selector(nsMenuX_NSMenu_removeItem:fromTable:), PR_TRUE);
+      // On SnowLeopard the Shortcut framework (which contains the
+      // SCTGRLIndex class) is loaded on demand, whenever the user first opens
+      // a menu (which normally hasn't happened yet).  So we need to load it
+      // here explicitly.
+      if (nsToolkit::OnSnowLeopardOrLater())
+        dlopen("/System/Library/PrivateFrameworks/Shortcut.framework/Shortcut", RTLD_LAZY);
       Class SCTGRLIndexClass = ::NSClassFromString(@"SCTGRLIndex");
       nsToolkit::SwizzleMethods(SCTGRLIndexClass, @selector(indexMenuBarDynamically),
                                 @selector(nsMenuX_SCTGRLIndex_indexMenuBarDynamically));
@@ -114,17 +121,22 @@ nsMenuX::nsMenuX()
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
-
 nsMenuX::~nsMenuX()
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
 
+  // Prevent the icon object from outliving us.
+  if (mIcon)
+    mIcon->Destroy();
+
   RemoveAll();
 
   [mNativeMenu setDelegate:nil];
   [mNativeMenu release];
   [mMenuDelegate release];
-  [mNativeMenuItem release];
+  // autorelease the native menu item so that anything else happening to this
+  // object happens before the native menu item actually dies
+  [mNativeMenuItem autorelease];
 
   // alert the change notifier we don't care no more
   if (mContent)
@@ -135,7 +147,6 @@ nsMenuX::~nsMenuX()
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
-
 nsresult nsMenuX::Create(nsMenuObjectX* aParent, nsMenuBarX* aMenuBar, nsIContent* aNode)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
@@ -160,9 +171,8 @@ nsresult nsMenuX::Create(nsMenuObjectX* aParent, nsMenuBarX* aMenuBar, nsIConten
   if (mContent->GetChildCount() == 0)
     mVisible = PR_FALSE;
 
-  NSString *newCocoaLabelString = nsMenuUtilsX::CreateTruncatedCocoaLabel(mLabel);
+  NSString *newCocoaLabelString = nsMenuUtilsX::GetTruncatedCocoaLabel(mLabel);
   mNativeMenuItem = [[NSMenuItem alloc] initWithTitle:newCocoaLabelString action:nil keyEquivalent:@""];
-  [newCocoaLabelString release];
   [mNativeMenuItem setSubmenu:mNativeMenu];
 
   SetEnabled(!mContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::disabled,
@@ -181,7 +191,6 @@ nsresult nsMenuX::Create(nsMenuObjectX* aParent, nsMenuBarX* aMenuBar, nsIConten
   NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
 }
 
-
 nsresult nsMenuX::AddMenuItem(nsMenuItemX* aMenuItem)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
@@ -206,12 +215,33 @@ nsresult nsMenuX::AddMenuItem(nsMenuItemX* aMenuItem)
   // set its command. we get the unique command id from the menubar
   [newNativeMenuItem setTag:mMenuBar->RegisterForCommand(aMenuItem)];
 
+  // On SnowLeopard, our Help menu items often get disabled when the user
+  // enters a password after waking from sleep or the screen saver.  Whether
+  // or not the user is prompted for a password is governed by the "Require
+  // password" setting in the Security pref panel.  For more information see
+  // bugs 513048 and 530633, and the comments above similar code in
+  // MyMenuEventHandler() below and nsMenuItemX::ObserveAttributeChanged().
+  if (nsToolkit::OnSnowLeopardOrLater() && nsMenuX::IsXULHelpMenu(mContent)) {
+    MenuRef helpMenuRef = _NSGetCarbonMenu(mNativeMenu);
+    if (helpMenuRef) {
+      NSInteger index = [mNativeMenu indexOfItem:newNativeMenuItem];
+      BOOL cocoaEnabled = [newNativeMenuItem isEnabled];
+      Boolean carbonEnabled = ::IsMenuItemEnabled(helpMenuRef, index + 1);
+      if (carbonEnabled != cocoaEnabled) {
+        if (!carbonEnabled) {
+          ::EnableMenuItem(helpMenuRef, index + 1);
+        } else {
+          ::DisableMenuItem(helpMenuRef, index + 1);
+        }
+      }
+    }
+  }
+
   return NS_OK;
 
   NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
 }
 
-
 nsresult nsMenuX::AddMenu(nsMenuX* aMenu)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
@@ -238,14 +268,12 @@ nsresult nsMenuX::AddMenu(nsMenuX* aMenu)
   NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
 }
 
-
 // Includes all items, including hidden/collapsed ones
 PRUint32 nsMenuX::GetItemCount()
 {
   return mMenuObjectsArray.Length();
 }
 
-
 // Includes all items, including hidden/collapsed ones
 nsMenuObjectX* nsMenuX::GetItemAt(PRUint32 aPos)
 {
@@ -255,7 +283,6 @@ nsMenuObjectX* nsMenuX::GetItemAt(PRUint32 aPos)
   return mMenuObjectsArray[aPos];
 }
 
-
 // Only includes visible items
 nsresult nsMenuX::GetVisibleItemCount(PRUint32 &aCount)
 {
@@ -263,7 +290,6 @@ nsresult nsMenuX::GetVisibleItemCount(PRUint32 &aCount)
   return NS_OK;
 }
 
-
 // Only includes visible items. Note that this is provides O(N) access
 // If you need to iterate or search, consider using GetItemAt and doing your own filtering
 nsMenuObjectX* nsMenuX::GetVisibleItemAt(PRUint32 aPos)
@@ -294,7 +320,6 @@ nsMenuObjectX* nsMenuX::GetVisibleItemAt(PRUint32 aPos)
   return NULL;
 }
 
-
 nsresult nsMenuX::RemoveAll()
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
@@ -317,7 +342,6 @@ nsresult nsMenuX::RemoveAll()
   NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
 }
 
-
 nsEventStatus nsMenuX::MenuOpened(const nsMenuEvent & aMenuEvent)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
@@ -367,7 +391,6 @@ nsEventStatus nsMenuX::MenuOpened(const nsMenuEvent & aMenuEvent)
   NS_OBJC_END_TRY_ABORT_BLOCK_RETURN(nsEventStatus_eIgnore);
 }
 
-
 void nsMenuX::MenuClosed(const nsMenuEvent & aMenuEvent)
 {
   if (mConstructed) {
@@ -386,7 +409,6 @@ void nsMenuX::MenuClosed(const nsMenuEvent & aMenuEvent)
   }
 }
 
-
 void nsMenuX::MenuConstruct()
 {
   mConstructed = false;
@@ -449,14 +471,12 @@ void nsMenuX::MenuConstruct()
   // printf("Done building, mMenuObjectsArray.Count() = %d \n", mMenuObjectsArray.Count());
 }
 
-
 void nsMenuX::SetRebuild(PRBool aNeedsRebuild)
 {
   if (!gConstructingMenu)
     mNeedsRebuild = aNeedsRebuild;
 }
 
-
 nsresult nsMenuX::SetEnabled(PRBool aIsEnabled)
 {
   if (aIsEnabled != mIsEnabled) {
@@ -467,7 +487,6 @@ nsresult nsMenuX::SetEnabled(PRBool aIsEnabled)
   return NS_OK;
 }
 
-
 nsresult nsMenuX::GetEnabled(PRBool* aIsEnabled)
 {
   NS_ENSURE_ARG_POINTER(aIsEnabled);
@@ -475,7 +494,6 @@ nsresult nsMenuX::GetEnabled(PRBool* aIsEnabled)
   return NS_OK;
 }
 
-
 GeckoNSMenu* nsMenuX::CreateMenuWithGeckoString(nsString& menuTitle)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NIL;
@@ -498,7 +516,6 @@ GeckoNSMenu* nsMenuX::CreateMenuWithGeckoString(nsString& menuTitle)
   NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
 }
 
-
 void nsMenuX::LoadMenuItem(nsIContent* inMenuItemContent)
 {
   if (!inMenuItemContent)
@@ -541,7 +558,6 @@ void nsMenuX::LoadMenuItem(nsIContent* inMenuItemContent)
   menuItem->SetupIcon();
 }
 
-
 void nsMenuX::LoadSubMenu(nsIContent* inMenuContent)
 {
   nsAutoPtr<nsMenuX> menu(new nsMenuX());
@@ -561,7 +577,6 @@ void nsMenuX::LoadSubMenu(nsIContent* inMenuContent)
   menu.forget();
 }
 
-
 // This menu is about to open. Returns TRUE if we should keep processing the event,
 // FALSE if the handler wants to stop the opening of the menu.
 PRBool nsMenuX::OnOpen()
@@ -644,7 +659,6 @@ PRBool nsMenuX::OnOpen()
   return PR_TRUE;
 }
 
-
 PRBool nsMenuX::OnOpened()
 {
   nsEventStatus status = nsEventStatus_eIgnore;
@@ -662,7 +676,6 @@ PRBool nsMenuX::OnOpened()
   return PR_TRUE;
 }
 
-
 // Returns TRUE if we should keep processing the event, FALSE if the handler
 // wants to stop the closing of the menu.
 PRBool nsMenuX::OnClose()
@@ -689,7 +702,6 @@ PRBool nsMenuX::OnClose()
   return PR_TRUE;
 }
 
-
 PRBool nsMenuX::OnClosed()
 {
   nsEventStatus status = nsEventStatus_eIgnore;
@@ -711,7 +723,6 @@ PRBool nsMenuX::OnClosed()
   return PR_TRUE;
 }
 
-
 // Find the |menupopup| child in the |popup| representing this menu. It should be one
 // of a very few children so we won't be iterating over a bazillion menu items to find
 // it (so the strcmp won't kill us).
@@ -739,21 +750,29 @@ void nsMenuX::GetMenuPopupContent(nsIContent** aResult)
       return;
     }
   }
-
 }
 
-
 NSMenuItem* nsMenuX::NativeMenuItem()
 {
   return mNativeMenuItem;
 }
 
+PRBool nsMenuX::IsXULHelpMenu(nsIContent* aMenuContent)
+{
+  PRBool retval = PR_FALSE;
+  if (aMenuContent) {
+    nsAutoString id;
+    aMenuContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::id, id);
+    if (id.Equals(NS_LITERAL_STRING("helpMenu")))
+      retval = PR_TRUE;
+  }
+  return retval;
+}
 
 //
 // nsChangeObserver
 //
 
-
 void nsMenuX::ObserveAttributeChanged(nsIDocument *aDocument, nsIContent *aContent,
                                       nsIAtom *aAttribute)
 {
@@ -778,9 +797,8 @@ void nsMenuX::ObserveAttributeChanged(nsIDocument *aDocument, nsIContent *aConte
     if (parentType == eMenuBarObjectType) {
       // reuse the existing menu, to avoid rebuilding the root menu bar.
       NS_ASSERTION(mNativeMenu, "nsMenuX::AttributeChanged: invalid menu handle.");
-      NSString *newCocoaLabelString = nsMenuUtilsX::CreateTruncatedCocoaLabel(mLabel);
+      NSString *newCocoaLabelString = nsMenuUtilsX::GetTruncatedCocoaLabel(mLabel);
       [mNativeMenu setTitle:newCocoaLabelString];
-      [newCocoaLabelString release];
     }
     else {
       static_cast<nsMenuX*>(mParent)->SetRebuild(PR_TRUE);
@@ -829,7 +847,6 @@ void nsMenuX::ObserveAttributeChanged(nsIDocument *aDocument, nsIContent *aConte
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
-
 void nsMenuX::ObserveContentRemoved(nsIDocument *aDocument, nsIContent *aChild,
                                     PRInt32 aIndexInContainer)
 {
@@ -840,7 +857,6 @@ void nsMenuX::ObserveContentRemoved(nsIDocument *aDocument, nsIContent *aChild,
   mMenuBar->UnregisterForContentChanges(aChild);
 }
 
-
 void nsMenuX::ObserveContentInserted(nsIDocument *aDocument, nsIContent *aChild,
                                      PRInt32 aIndexInContainer)
 {
@@ -850,7 +866,6 @@ void nsMenuX::ObserveContentInserted(nsIDocument *aDocument, nsIContent *aChild,
   SetRebuild(PR_TRUE);
 }
 
-
 nsresult nsMenuX::SetupIcon()
 {
   // In addition to out-of-memory, menus that are children of the menu bar
@@ -861,12 +876,10 @@ nsresult nsMenuX::SetupIcon()
   return mIcon->SetupIcon();
 }
 
-
 //
 // Carbon event support
 //
 
-
 static pascal OSStatus MyMenuEventHandler(EventHandlerCallRef myHandler, EventRef event, void* userData)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
@@ -883,6 +896,46 @@ static pascal OSStatus MyMenuEventHandler(EventHandlerCallRef myHandler, EventRe
 
   nsMenuX* targetMenu = static_cast<nsMenuX*>(userData);
   UInt32 kind = ::GetEventKind(event);
+
+  // On SnowLeopard, our Help menu items often get disabled when the user
+  // enters a password after waking from sleep or the screen saver.  Whether
+  // or not the user is prompted for a password is governed by the "Require
+  // password" setting in the Security pref panel.  For more information see
+  // bug 513048.
+  //
+  // This is surely an OS bug, though it's not clear exactly what triggers it.
+  // The end result is that the Help menu's items are turned off in Carbon,
+  // even though they're turned on in Cocoa.  (On SnowLeopard, system menus
+  // are still implemented in Carbon (at least for for 32-bit apps), using the
+  // undocumented NSCarbonMenuImpl class.)  The workaround for this is to do
+  // the following whenever the Help menu is opened, or a Help menu item is
+  // added or changed:  Check the Carbon and Cocoa enabled states of each menu
+  // item.  If the states don't match, change the Carbon enabled state to
+  // match the Cocoa enabled state.
+  if (nsToolkit::OnSnowLeopardOrLater() && (kind == kEventMenuOpening) &&
+      targetMenu && nsMenuX::IsXULHelpMenu(targetMenu->Content())) {
+    NSMenu *nativeMenu = static_cast<NSMenu*>(targetMenu->NativeData());
+    if (nativeMenu) {
+      MenuRef helpMenuRef = _NSGetCarbonMenu(nativeMenu);
+      if (helpMenuRef) {
+        NSArray *items = [nativeMenu itemArray];
+        NSUInteger count = [items count];
+        for (NSUInteger i = 0; i < count; ++i) {
+          NSMenuItem *anItem = (NSMenuItem *) [items objectAtIndex:i];
+          BOOL cocoaEnabled = [anItem isEnabled];
+          Boolean carbonEnabled = ::IsMenuItemEnabled(helpMenuRef, i + 1);
+          if (carbonEnabled != cocoaEnabled) {
+            if (!carbonEnabled) {
+              ::EnableMenuItem(helpMenuRef, i + 1);
+            } else {
+              ::DisableMenuItem(helpMenuRef, i + 1);
+            }
+          }
+        }
+      }
+    }
+  }
+
   if (kind == kEventMenuTargetItem) {
     // get the position of the menu item we want
     PRUint16 aPos;
@@ -891,13 +944,11 @@ static pascal OSStatus MyMenuEventHandler(EventHandlerCallRef myHandler, EventRe
     
     // don't request a menu item that doesn't exist or we crash
     // this might happen just due to some random quirks in the event system
-    PRUint32 itemCount;
-    targetMenu->GetVisibleItemCount(itemCount);
-    if (aPos >= itemCount)
+    nsMenuObjectX* target = targetMenu->GetVisibleItemAt((PRUint32)aPos);
+    if (!target)
       return eventNotHandledErr;
 
     // Send DOM event if we're over a menu item
-    nsMenuObjectX* target = targetMenu->GetVisibleItemAt((PRUint32)aPos);
     if (target->MenuObjectType() == eMenuItemObjectType) {
       nsMenuItemX* targetMenuItem = static_cast<nsMenuItemX*>(target);
       PRBool handlerCalledPreventDefault; // but we don't actually care
@@ -907,7 +958,7 @@ static pascal OSStatus MyMenuEventHandler(EventHandlerCallRef myHandler, EventRe
   }
   else if (kind == kEventMenuOpening || kind == kEventMenuClosed) {
     if (kind == kEventMenuOpening && gRollupListener && gRollupWidget) {
-      gRollupListener->Rollup(nsnull);
+      gRollupListener->Rollup(nsnull, nsnull);
       return userCanceledErr;
     }
     MenuRef menuRef;
@@ -926,7 +977,6 @@ static pascal OSStatus MyMenuEventHandler(EventHandlerCallRef myHandler, EventRe
   NS_OBJC_END_TRY_ABORT_BLOCK_RETURN(noErr);
 }
 
-
 static OSStatus InstallMyMenuEventHandler(MenuRef menuRef, void* userData, EventHandlerRef* outHandler)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
@@ -947,15 +997,12 @@ static OSStatus InstallMyMenuEventHandler(MenuRef menuRef, void* userData, Event
   NS_OBJC_END_TRY_ABORT_BLOCK_RETURN(noErr);
 }
 
-
 //
 // MenuDelegate Objective-C class, used to set up Carbon events
 //
 
-
 @implementation MenuDelegate
 
-
 - (id)initWithGeckoMenu:(nsMenuX*)geckoMenu
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NIL;
@@ -980,7 +1027,6 @@ static OSStatus InstallMyMenuEventHandler(MenuRef menuRef, void* userData, Event
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
-
 // You can get a MenuRef from an NSMenu*, but not until it has been made visible
 // or added to the main menu bar. Basically, Cocoa is attempting lazy loading,
 // and that doesn't work for us. We don't need any carbon events until after the
@@ -1004,7 +1050,6 @@ static OSStatus InstallMyMenuEventHandler(MenuRef menuRef, void* userData, Event
 
 @end
 
-
 // OS X Leopard (at least as of 10.5.2) has an obscure bug triggered by some
 // behavior that's present in Mozilla.org browsers but not (as best I can
 // tell) in Apple products like Safari.  (It's not yet clear exactly what this
@@ -1119,7 +1164,6 @@ static NSMutableDictionary *gShadowKeyEquivDB = nil;
 
 @end
 
-
 @interface NSMenu (MethodSwizzling)
 + (void)nsMenuX_NSMenu_addItem:(NSMenuItem *)aItem toTable:(NSMapTable *)aTable;
 + (void)nsMenuX_NSMenu_removeItem:(NSMenuItem *)aItem fromTable:(NSMapTable *)aTable;
